/*
 * Copyright (c) 2010-2011 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.kex.ftp.DavKommunikation
 *
 * de.inovat.kex.ftp.DavKommunikation is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.kex.ftp.DavKommunikation is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.kex.ftp.DavKommunikation; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.kex.ftp;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.config.*;
import de.bsvrz.sys.funclib.debug.Debug;

import java.util.Calendar;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Die Klasse dient verwaltet die Datenverteiler Kommunikation. Das Anmelden, Abmelden und Senden der Daten.
 * <p/>
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dagmar Rostek (DR)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class DavKommunikation implements ClientSenderInterface {

    /** Aspekt Zustand. */
    private static final String ASP_ZUSTAND = "asp.zustand";

    /** Attributgruppe bastImportVerzeichnisberwachung. */
    private static final String ATG_VEREICHNISUEBERWACHUNG = "atg.bastImportVerzeichnisUeberwachung";

    /** DebugLogger fr Debug-Ausgaben. */
    private static final Debug debug = Debug.getLogger();

    //~ FELDER ================================================================

    /** AttributeGroup der Verzeichnisberwachungsdaten. */
    private AttributeGroup _atgVerzeichnisUeberwachung;

    /** Datenbeschreibung zum Senden der Verzeichnisberwachungsdaten. */
    private DataDescription _datenbeschreibungVerzeichnisUeberwachung;

    /** ClientDavInteface. */
    private ClientDavInterface _dav;

    /** SystemObjekt Verzeichnisberwachung. */
    private SystemObject _soVerzeichnisUeberwachung;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erstelle Instanz der Klasse.
     *
     * @param dav ClientDavInterface.
     * @param typBAStBand Typ des Systemobjekte fr den BASt-Band Import/Export.
     */
    public DavKommunikation(ClientDavInterface dav, String typBAStBand) {
        _dav = dav;

        // Initialisiere DaV-Daten:
        _soVerzeichnisUeberwachung  = _dav.getDataModel().getObject(typBAStBand);
        _atgVerzeichnisUeberwachung = _dav.getDataModel().getAttributeGroup(ATG_VEREICHNISUEBERWACHUNG);

        Aspect aspektZustand = _dav.getDataModel().getAspect(ASP_ZUSTAND);

        _datenbeschreibungVerzeichnisUeberwachung = new DataDescription(_atgVerzeichnisUeberwachung, aspektZustand);
    }

    //~ METHODEN ==============================================================

    /**
     * Abmelden vom Datenverteiler.
     */
    private void abmeldenAlsDaVSender() {
        _dav.unsubscribeSender(this, _soVerzeichnisUeberwachung, _datenbeschreibungVerzeichnisUeberwachung);
    }

    /**
     * Anmeldung beim Datenverteiler als Sender fr den Status bzgl. Senden der BASt-Band Daten.
     *
     * @param dateiName Dateiname, fr den die Anmeldung erfolgt (nur fr die Debugausgabe)
     */
    private void anmeldenAlsDavSender(String dateiName) {

        // Anmelden zum Senden der Daten
        subscribe(_soVerzeichnisUeberwachung, _datenbeschreibungVerzeichnisUeberwachung, SenderRole.sender());

        //
        debug.info("Die Anmeldung beim Datenverteiler zum Senden der Verzeichnisinformationsdaten ist erfolgt fuer die Datei: " + dateiName);
    }

    /**
     * Die Methode bestimmt das BASt-Band Format abhngig vom berwachungsverzeichnis.
     *
     * @param ueberwachungsVerezichnis  berwachungsverzeichnis.
     *
     * @return integer Zahl die das Format reprsentiert. Return Wert von 0 entspricht dem BASt-Band Format 2004,
     * Return Wert von 5 entspricht dem BASt-Band Format VerkehrGeschwindigkeit2007. Sollten beide
     * Formate nicht mit dem bergebenem bereinstimmten wird -1 retuniert.
     */
    private int bestimmeBastFormat(String ueberwachungsVerezichnis) {
        if (ueberwachungsVerezichnis.contains("Verkehr_Menge_2004")) {
            return 0;
        } else if (ueberwachungsVerezichnis.contains("Verkehr_Geschwindigkeit_2007")) {
            return 5;
        } else {
            return -1;
        }
    }

    // @Override
    public void dataRequest(SystemObject systemObject, DataDescription dataDescription, byte b) {}

    //~ GET METHODEN ==========================================================

    // @Override
    public boolean isRequestSupported(SystemObject systemObject, DataDescription dataDescription) {
        return false;
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode sendet das Verzeichnis samt Datei.
     *
     * @param dateiName  Dateiname mit dem Pfad relativ zum Server-Root-Ordner.
     * @param istFtp Definiert den ServerTyp: FTP (true) oder SFTP (false).
     * @param url URL.
     * @param port Port.
     * @param user  FTP user.
     * @param passwd  FTP login.
     * @param ftpPassivModus Stellt fest, ob passive FTP-Verbindung aufgebaut werden soll.
     * @param bastFormat  BASt-Format.
     */
    private void sendeDavDaten(String dateiName, boolean istFtp, String url, int port, String user, String passwd, boolean ftpPassivModus, int bastFormat) {

        // DaV-Datenformat:
        String passivModus                 = ftpPassivModus ? "ja" : "nein";
        String serverTyp                   = istFtp ? "FTP" : "SFTP";
        String allInfosAlsString           = String.format("%s@%s@%s@%s@%s@%s@%s@%s@", dateiName, url, port, user, passwd, passivModus, bastFormat, serverTyp);
        Data   dataVDatenLangZeitIntervall = _dav.createData(_atgVerzeichnisUeberwachung);
        long   zeitstempel                 = Calendar.getInstance().getTimeInMillis();

        dataVDatenLangZeitIntervall.getTextValue("ImportDatei").setText(allInfosAlsString);

        ResultData resultDate = new ResultData(_soVerzeichnisUeberwachung, _datenbeschreibungVerzeichnisUeberwachung, zeitstempel, dataVDatenLangZeitIntervall);

        try {
            _dav.sendData(resultDate);
        }
        catch (SendSubscriptionNotConfirmed sendSubscriptionNotConfirmed) {
            debug.warning("Die Daten konnten nicht versendet werden! Bitte kontrollieren Sie ob der BASt-Band Prozess luft! " + sendSubscriptionNotConfirmed);
        }
    }

    /**
     * Die Methode sendet das Verzeichnis samt Datei.
     *
     * @param dateiName  Dateiname mit dem Pfad relativ zum Server-Root-Ordner.
     * @param ueberwachungsVerezichnis  berwachungsverzeichnis um BastBandFormat zu bestimmen.
     * @param istFtp Definiert den ServerTyp: FTP (true) oder SFTP (false).
     * @param url URL.
     * @param port Port.
     * @param user  FTP user.
     * @param passwd  FTP login.
     * @param ftpPassivModus Stellt fest, ob passive FTP-Verbindung aufgebaut werden soll.
     */
    public void sendeVerzeichnisUeberwachungsDaVDaten(String dateiName, String ueberwachungsVerezichnis, boolean istFtp, String url, int port, String user, String passwd, boolean ftpPassivModus) {
        anmeldenAlsDavSender(dateiName);

        int bastFormat = bestimmeBastFormat(ueberwachungsVerezichnis);

        if (bastFormat == -1) {
            String infoText = "Das berwachungsverzeichnis ist nicht korrekt, bitte berprfen Sie ihre Angaben.";

            debug.error(infoText);
        } else {
            sendeDavDaten(dateiName, istFtp, url, port, user, passwd, ftpPassivModus, bastFormat);
            abmeldenAlsDaVSender();
        }
    }

    /**
     * Anmeldung beim Datenverteiler.
     * @param systemObjectVerzeichnisUeberwachung  SystemObject fr die Verzeichnisberwachung.
     * @param datenbeschreibungVerzeichnisUeberwachung  Datenbeschreibung.
     * @param senderRolle Senderrolle.
     */
    private void subscribe(SystemObject systemObjectVerzeichnisUeberwachung, DataDescription datenbeschreibungVerzeichnisUeberwachung, SenderRole senderRolle) {
        try {
            _dav.subscribeSender(this, systemObjectVerzeichnisUeberwachung, datenbeschreibungVerzeichnisUeberwachung, senderRolle);
        }
        catch (OneSubscriptionPerSendData oneSubscriptionPerSendData) {
            String infoText = "Es konnte keine Anmeldung beim Datenverteiler vollzogen werden." + oneSubscriptionPerSendData;

            debug.error(infoText);
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 27.06.13
